# This file is part of SSHare.
#
# SSHare is free software: you can redistribute it and/or modify it under the terms of
# the GNU General Public License as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
#
# SSHare is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# SSHare. If not, see <https://www.gnu.org/licenses/>.

class NoDefault: pass

def Flag(name=None, help=None):
    return Argument(
        name,
        action="store_const",
        const=Flag,
        default=False,
        help=help,
    )

class Argument:
    def __init__(self, name=None, **kwargs):
        class _None:
            def __init__(self, default):
                self.default = default
        self._None = _None

        self._short = None
        self._long  = name

        if not "default" in kwargs:
            kwargs["default"] = NoDefault
        kwargs["default"] = _None(kwargs["default"])
        self._kwargs = kwargs

    def bind(self, plugin, argument):
        self._plugin = plugin.name
        if self._long == None:
            self._long = argument
        self._kwargs["metavar"] = argument

    def set_flags(self, short, long):
        if short != None:
            if short == False:
                self._short = None
            else:
                self._short = short
        if long != None:
            if long == False:
                self._long = None
            else:
                self._long = long

    def default(self):
        value = self._kwargs["default"]
        if isinstance(value, self._None):
            value = value.default
        return value

    def dest(self):
        return f"{self._plugin}_{self._kwargs["metavar"]}"

    def extract(self, arguments):
        value = getattr(arguments, self.dest())
        was_set = True
        if isinstance(value, self._None):
            was_set = False
            value = value.default
        return was_set, value

    def add(self, arg_parser):
        flags = []
        if self._short != None: flags.append(f"-{self._short}")
        if self._long  != None: flags.append(f"--{self._long}")
        kwargs = self._kwargs | {
            "dest": self.dest()
        }
        arg_parser.add_argument(
            *flags,
            **kwargs
        )
