# This file is part of SSHare.
#
# SSHare is free software: you can redistribute it and/or modify it under the terms of
# the GNU General Public License as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
#
# SSHare is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# SSHare. If not, see <https://www.gnu.org/licenses/>.

import time

from sshare.plugins.config import Argument
from sshare.plugins.source import File

plugin_type = "name"

config = {
    "base": 62,
}

def init():
    if not isinstance(config.base, int):
        logger.fatal("Error: 'base' must be an integer")
    elif config.base < 2:
        logger.fatal("Error: 'base' cannot be less than 2")
    elif config.base > 62:
        logger.fatal("Error: 'base' cannot be greater than 62")
    
def get_name(name, source):
    return name + _rebase(config.base, time.time_ns())

def _rebase(base, number):
    if number == 0:
        return "0"
    if base == 10:
        return f"{number}"
    rebased = ""
    while number != 0:
        rebased = _number_to_char(number % base) + rebased
        number  = int(number / base)
    return rebased

def _number_to_char(number):
    if number < 10:
        return chr(number + 48)
    elif number < 36:
        return chr(number + 87)
    else:
        return chr(number + 29)
